import importlib
import os
import sys
import json
import time
from fastapi.staticfiles import StaticFiles
import shutil
import importlib.machinery
import importlib.util

import databutton as db
from databutton.helpers import parse
from databutton.server.processes import start_processes
from fastapi import FastAPI
from starlette.responses import RedirectResponse

app = FastAPI()
sys.path.insert(0, os.curdir)
sys.path.append(os.curdir + '/.databutton')

# Create .databutton if not exists
if not os.path.exists('.databutton'):
    os.mkdir('.databutton')

# Find all directive modules and import them
imports = parse.find_databutton_directive_modules()

# Clean the existing artifacts, generate new one
# TODO: Have a cache mechanism to improve performance
shutil.rmtree('.databutton/app', ignore_errors=True)
shutil.rmtree('.databutton/data', ignore_errors=True)
decorator_modules = {}
for name in imports:
    decorator_modules[name] = importlib.import_module(name)


# Generate routes for dataframes
data_modules = {}
for dataframe in db._dataframes:
    name = dataframe.modulename
    file = dataframe.importfile
    spec = importlib.util.spec_from_file_location(name, file)
    data_modules[name] = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(data_modules[name])
    app.include_router(data_modules[name].router)

# Write db._x to artifacts.json
artifacts = {
    'streamlit_apps': [{'route': k, 'filepath': v, 'last_updated': int(time.time() * 1000)} for k, v in db._streamlit_apps.items()],
    'functions': [],
    'routes': [],
    'dataframes': [df.__dict__ for df in db._dataframes]
}

with open('.databutton/artifacts.json', 'w') as f:
    json.dump(artifacts, f)

app.mount('/static', StaticFiles(directory='.databutton'), name='.databutton')

dir_path = os.path.dirname(os.path.realpath(__file__))
app_dir = os.path.join(dir_path, '../out')


@app.on_event('startup')
async def start_servers():
    print('Starting processes')
    await start_processes(app, db._streamlit_apps)

    @app.get('/')
    async def index():
        return RedirectResponse(url='/index.html')

    app.mount('/', StaticFiles(directory=app_dir), name='app')
