import importlib
import json
import os
from pathlib import Path
import shutil
import sys
import tarfile
import time
import yaml

import databutton as db
from databutton.helpers import parse
from databutton.utils.kubernetes import generate_manifest


def generate_components():
    sys.path.append(os.curdir)
    sys.path.append(os.curdir + '/.databutton')
    # Find all directive modules and import them
    imports = parse.find_databutton_directive_modules()

    # Clean the existing artifacts, generate new one
    # TODO: Have a cache mechanism to improve performance
    shutil.rmtree('.databutton', ignore_errors=True)
    decorator_modules = {}
    for name in imports:
        decorator_modules[name] = importlib.import_module(name)

    # Write the artifacts
    artifacts = {
        'streamlit_apps': [{
            'route': k,
            'filepath': v,
            'last_updated': int(time.time() * 1000),
            'port': 8501 + i,
        }
            for i, (k, v) in enumerate(db._streamlit_apps.items())
        ],
        'functions': [],
        'routes': [],
        'dataframes': [df.__dict__ for df in db._dataframes]
    }

    with open('.databutton/artifacts.json', 'w') as f:
        json.dump(artifacts, f)

    # Copy the Dockerfile
    dir_path = os.path.dirname(os.path.realpath(__file__))
    shutil.copytree(os.path.join(dir_path, '../docker'), '.databutton/docker')

    # Generate a kubernetes manifest
    manifest = generate_manifest(artifacts)
    kube_path = '.databutton/kubernetes'
    Path(kube_path).mkdir(exist_ok=True, parents=True)
    for kind, d in manifest.items():
        with open(os.path.join(kube_path, f"{kind}.yaml"), 'w') as f:
            yaml.safe_dump(d, f, allow_unicode=True)


def create_archive(name: str, source_dir: str = os.curdir):
    with tarfile.open(name, "w:gz") as tar:
        for fn in os.listdir(source_dir):
            p = os.path.join(source_dir, fn)
            tar.add(p, arcname=fn)
