from time import sleep
from databutton.utils import get_auth_token
import requests


token = None


def get_firebase(path: str):
    global token
    if token is None:
        token = get_auth_token()
    return requests.get(
        f"https://firestore.googleapis.com/v1/projects/databutton/databases/(default)/documents/{path}",
        headers={'Authorization': f"Bearer {token}"}
    )


def wait_for_firebase_existence(path, retries=10):
    res = get_firebase(path)
    if res.status_code == 404:
        # This doesn't exist (yet), try again in 1 second
        sleep(5)
        return wait_for_firebase_existence(path)
    return res


def get_deployment(deployment_id):
    return wait_for_firebase_existence(f"deployments/{deployment_id}").json()


def get_build_status(build_id: str):
    json = get_firebase(f"cloud-builds/{build_id}").json()
    return json['fields']['status']['stringValue']


def listen_to_build(deployment_id: str):
    deployment = get_deployment(deployment_id)
    build_id = deployment['fields']['buildId']['stringValue']
    status = get_build_status(build_id)
    while status != 'SUCCESS':
        sleep(5)
        status = get_build_status(build_id)
    return status
