from typing import List


def generate_manifest(components: List[any]):
    deployment = {
        'apiVersion': 'apps/v1',
        'kind': 'Deployment',
        'metadata': {
            'name': 'PROJECT_ID',
            'labels': {
                'app': 'PROJECT_ID'
            }
        },
        'spec': {
            'replicas': 1,
            'selector': {
                'matchLabels': {
                    'app': 'PROJECT_ID'
                }
            },
            'template': {
                'metadata': {
                    'labels': {
                        'project_id': 'PROJECT_ID',
                        'app': 'PROJECT_ID'
                    }
                },
                'spec': {
                    'containers': [
                        {
                            'name': f"route-{app['route'].replace('/', '-')}",
                            'image': 'IMAGE_NAME',
                            'args': [app['filepath'], '--server.port', f"{app['port']}", '--server.address', "0.0.0.0"],
                            'ports': [{'containerPort': app['port']}],
                            'env': [
                                {
                                    'name': 'PYTHONPATH',
                                    'value': '.'
                                }
                            ],
                            'command': ['streamlit', 'run'],
                            'resources': {
                                'requests': {
                                    'memory': '256Mi',
                                    'cpu': '0.4'
                                },
                                'limits': {
                                    'memory': '512Mi',
                                    'cpu': '0.8'
                                }
                            }
                        }
                        for app in components['streamlit_apps']
                    ] + [{
                        'name': 'main',
                        'image': 'IMAGE_NAME',
                        'ports': [{
                            'containerPort': 8000
                        }],
                        'command': ['databutton', 'serve'],
                        'resources': {
                            'requests': {
                                'memory': '256Mi',
                                'cpu': '0.4'
                            },
                            'limits': {
                                'memory': '512Mi',
                                'cpu': '0.8'
                            }
                        }
                    }]
                }
            }
        }
    }

    mapping = {
        'apiVersion': 'getambassador.io/v3alpha1',
        'kind': 'Mapping',
        'metadata': {
            'name': 'PROJECT_ID',
        },
        'spec': {
            'hostname': 'PROJECT_ID.dbtn.app',
            'service': 'PROJECT_ID',
            'prefix': '/',
            'allow_upgrade': ['websocket']
        }
    }

    service = {
        'apiVersion': 'v1',
        'kind': 'Service',
        'metadata': {
            'name': 'PROJECT_ID',
        },
        'spec': {
            'ports': [{
                'port': 80,
                'targetPort': 8000
            }],
            'selector': {
                'app': 'PROJECT_ID'
            }
        }
    }

    return {
        'deployment': deployment,
        'service': service,
        'mapping': mapping
    }
